import { Span } from 'opentracing';
import { FileSystemUpdater } from './fs';
import { Logger } from './logging';
import { InMemoryFileSystem } from './memfs';
/**
 * Schema of a package.json file
 */
export interface PackageJson {
    name: string;
    version?: string;
    typings?: string;
    repository?: string | {
        type: string;
        url: string;
    };
    dependencies?: {
        [packageName: string]: string;
    };
    devDependencies?: {
        [packageName: string]: string;
    };
    peerDependencies?: {
        [packageName: string]: string;
    };
    optionalDependencies?: {
        [packageName: string]: string;
    };
}
export declare class PackageManager {
    private updater;
    private inMemoryFileSystem;
    private logger;
    /**
     * True if the workspace was scanned for package.json files, they were fetched and parsed
     */
    private scanned;
    /**
     * Set of package.json URIs _defined_ in the workspace.
     * This does not include package.jsons of dependencies and also not package.jsons that node_modules are vendored for
     */
    private packages;
    /**
     * The URI of the root package.json, if any
     */
    rootPackageJsonUri: string | undefined;
    constructor(updater: FileSystemUpdater, inMemoryFileSystem: InMemoryFileSystem, logger?: Logger);
    /**
     * Returns an Iterable for all package.jsons in the workspace
     */
    packageJsonUris(): IterableIterator<string>;
    /**
     * Scans the workspace to find all packages _defined_ in the workspace, saves the content in `packages`
     * For each found package, installation is started in the background and tracked in `installations`
     *
     * @param span OpenTracing span for tracing
     */
    private scan(span?);
    /**
     * Ensures all package.json have been detected, loaded and installations kicked off
     *
     * @param span OpenTracing span for tracing
     */
    ensureScanned(span?: Span): Promise<void>;
    /**
     * Gets the content of the closest package.json known to to the DependencyManager in the ancestors of a URI
     */
    getClosestPackageJson(uri: string, span?: Span): Promise<PackageJson | undefined>;
    /**
     * Walks the parent directories of a given URI to find the first package.json that is known to the InMemoryFileSystem
     *
     * @param uri URI of a file or directory in the workspace
     * @return The found package.json or undefined if none found
     */
    getClosestPackageJsonUri(uri: string): string | undefined;
}
