"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const opentracing_1 = require("opentracing");
const path = require("path");
const url = require("url");
const logging_1 = require("./logging");
class PackageManager {
    constructor(updater, inMemoryFileSystem, logger = new logging_1.NoopLogger()) {
        this.updater = updater;
        this.inMemoryFileSystem = inMemoryFileSystem;
        this.logger = logger;
        /**
         * True if the workspace was scanned for package.json files, they were fetched and parsed
         */
        this.scanned = false;
        /**
         * Set of package.json URIs _defined_ in the workspace.
         * This does not include package.jsons of dependencies and also not package.jsons that node_modules are vendored for
         */
        this.packages = new Set();
    }
    /**
     * Returns an Iterable for all package.jsons in the workspace
     */
    packageJsonUris() {
        return this.packages.values();
    }
    /**
     * Scans the workspace to find all packages _defined_ in the workspace, saves the content in `packages`
     * For each found package, installation is started in the background and tracked in `installations`
     *
     * @param span OpenTracing span for tracing
     */
    scan(span = new opentracing_1.Span()) {
        // Find locations of package.json and node_modules folders
        const packageJsons = new Set();
        let rootPackageJsonUri;
        let rootPackageJsonLevel = Infinity;
        for (const uri of this.inMemoryFileSystem.uris()) {
            const parts = url.parse(uri);
            if (!parts.pathname) {
                continue;
            }
            // Search for package.json files _not_ inside node_modules
            if (parts.pathname.endsWith('/package.json') && !parts.pathname.includes('/node_modules/')) {
                packageJsons.add(uri);
                // If the current root package.json is further nested than this one, replace it
                const level = parts.pathname.split('/').length;
                if (level < rootPackageJsonLevel) {
                    rootPackageJsonUri = uri;
                    rootPackageJsonLevel = level;
                }
            }
        }
        this.rootPackageJsonUri = rootPackageJsonUri;
        this.logger.log(`Found ${packageJsons.size} package.json in workspace`);
        this.logger.log(`Root package.json: ${rootPackageJsonUri}`);
        this.packages.clear();
        for (const uri of packageJsons) {
            this.packages.add(uri);
        }
        this.scanned = true;
    }
    /**
     * Ensures all package.json have been detected, loaded and installations kicked off
     *
     * @param span OpenTracing span for tracing
     */
    ensureScanned(span = new opentracing_1.Span()) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.updater.ensureStructure(span);
            if (!this.scanned) {
                this.scan(span);
            }
        });
    }
    /**
     * Gets the content of the closest package.json known to to the DependencyManager in the ancestors of a URI
     */
    getClosestPackageJson(uri, span = new opentracing_1.Span()) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.ensureScanned(span);
            const packageJsonUri = this.getClosestPackageJsonUri(uri);
            if (!packageJsonUri) {
                return undefined;
            }
            this.updater.ensure(packageJsonUri, span);
            return JSON.parse(yield this.inMemoryFileSystem.getContent(packageJsonUri));
        });
    }
    /**
     * Walks the parent directories of a given URI to find the first package.json that is known to the InMemoryFileSystem
     *
     * @param uri URI of a file or directory in the workspace
     * @return The found package.json or undefined if none found
     */
    getClosestPackageJsonUri(uri) {
        const parts = url.parse(uri);
        while (true) {
            if (!parts.pathname) {
                return undefined;
            }
            const packageJsonUri = url.format(Object.assign({}, parts, { pathname: path.posix.join(parts.pathname, 'package.json') }));
            if (this.packages.has(packageJsonUri)) {
                return packageJsonUri;
            }
            if (parts.pathname === '/') {
                return undefined;
            }
            parts.pathname = path.posix.dirname(parts.pathname);
        }
    }
}
exports.PackageManager = PackageManager;
//# sourceMappingURL=packages.js.map